#!/bin/bash
set -e

out=${1:-output}

echo "" > $out

echo "Below is the feedback for your submission of CW 7, Part 1 and 2." >> $out
echo "" >> $out


# compilation tests

function scala_compile {
  (ulimit -t 30; JAVA_OPTS="-Xmx1g" scala "$1" 2>> $out 1>> $out) 
}

# functional tests

function scala_assert_slow {
  (ulimit -t 120; JAVA_OPTS="-Xmx1g" scala -i "$1" "$2" -e "" 2> /dev/null 1> /dev/null)
}

function scala_assert_thirty {
  (ulimit -t 30; JAVA_OPTS="-Xmx1g" scala -i "$1" "$2" -e "" 2> /dev/null 1> /dev/null)
}

function scala_assert_quick {
  (ulimit -t 10; JAVA_OPTS="-Xmx1g" scala -i "$1" "$2" -e "" 2> /dev/null 1> /dev/null)
}

# purity test

function scala_vars {
   (egrep '\bvar\b|\breturn\b|\.par|ListBuffer|mutable|new Array' "$1" 2> /dev/null 1> /dev/null)
}


# knights1: purity test

echo "knight1.scala does not contain vars, returns etc?" >> $out

if (scala_vars knight1.scala)
then
  echo "  --> fail: if you do not fix this, you will receive a mark of zero" >> $out
  tsts0=$(( 1 ))
else
  echo "  --> success" >> $out
  tsts0=$(( 0 )) 
fi


# compilation test

if [ $tsts0 -eq 0 ]
then    
  echo "knight1.scala runs?" >> $out

  if (scala_compile knight1.scala)
  then
    echo "  --> success " >> $out
    tsts1=$(( 0 ))
  else
    echo "  --> scala did not run knight1.scala" >> $out
    tsts1=$(( 1 )) 
  fi
else
  tsts1=$(( 1 ))   
fi

### knight1a test

if [ $tsts1 -eq 0 ]
then
    echo "Takes 10 seconds or less to execute: " >> $out
    echo " is_legal(8, Nil)(3, 4) == true " >> $out
    echo " is_legal(8, List((4, 1), (1, 0)))(4, 1) == false " >> $out
    echo " is_legal(2, Nil)(0, 0) == true" >> $out                          

    if (scala_assert_quick "knight1.scala" "knight1a_test.scala")
    then
        echo "  --> success" >> $out
    else
        echo "  --> test failed" >> $out
    fi
fi

### knight1b test

if [ $tsts1 -eq 0 ]
then
  echo "Takes 10 seconds or less to execute: " >> $out
  echo " legal_moves(8, Nil, (2,2)) == List((3,4), (4,3), (4,1), (3,0), (1,0), (0,1), (0,3), (1,4))" >> $out
  echo " legal_moves(8, Nil, (7,7)) == List((6,5), (5,6))" >> $out
  echo " legal_moves(8, List((4,1), (1,0)), (2,2)) == List((3,4), (4,3), (3,0), (0,1), (0,3), (1,4))" >> $out
  echo " legal_moves(8, List((6,6)), (7,7)) == List((6,5), (5,6))" >> $out
  echo " legal_moves(1, Nil, (0,0)) == Nil" >> $out
  echo " legal_moves(2, Nil, (0,0)) == Nil" >> $out
  echo " legal_moves(3, Nil, (0,0)) == List((1,2), (2,1))" >> $out
  
  if (scala_assert_quick "knight1.scala" "knight1b_test.scala")
  then
    echo "  --> success" >> $out
  else
    echo "  --> test failed" >> $out
  fi
fi


### knight1c test

if [ $tsts1 -eq 0 ]
then
  echo " all_tours from every position on the board, in 2 minutes or less" >> $out
  echo " dim = 1: 1" >> $out
  echo "       2: 0,0,0,0" >>  $out
  echo "       3: 0,0,0,0,0,0,0,0,0" >>  $out
  echo "       4: 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0" >> $out
  echo "       5: 304,0,56,0,304,0,56,0,56,0,56,0,64,0,56,0,56,0,56,0,304,0,56,0,304" >> $out
  echo " enum_tours(5, List((0,2)) ) == 56 and all correct?" >> $out
  
  if (scala_assert_slow "knight1.scala" "knight1c_test.scala") 
  then
    echo "  --> success" >> $out
  else
    echo "  --> test failed" >> $out
  fi
fi


# knights2: var test

echo "knight2.scala does not contain vars, returns etc?" >> $out

if (scala_vars knight2.scala)
then
  echo "  --> fail" >> $out
  tsts0=$(( 1 ))
else
  echo "  --> success" >> $out
  tsts0=$(( 0 )) 
fi


# compilation test
if  [ $tsts0 -eq 0 ]
then    
  echo "knight2.scala runs?" >> $out

  if (scala_compile knight2.scala)
  then
    echo "  --> success" >> $out
    tsts1=$(( 0 ))
  else
    echo "  --> scala did not run knight2.scala" >> $out
    tsts1=$(( 1 )) 
  fi
else
  tsts1=$(( 1 ))     
fi

### knight2a test

if [ $tsts1 -eq 0 ]
then
  echo "Takes 10 seconds or less to execute: " >> $out
  echo " Let f = (x:(Int, Int)) => if (x._1 > 3) Some(List(x)) else None " >> $out
  echo "   first(List((1,0),(2,0),(3,0),(4,0)), f) == Some(List((4,0)))" >> $out
  echo "   first(List((1,0),(2,0),(3,0)), f) == None" >> $out

  if (scala_assert_quick "knight2.scala" "knight2a_test.scala") 
  then
    echo "  --> success" >> $out
  else
    echo "  --> test failed" >> $out
  fi
fi


### knight2b test

if [ $tsts1 -eq 0 ]
then
  echo "Takes 30 seconds or less to execute: " >> $out
  echo " is first_tour(8, List((0, 0))) ok? " >> $out
  echo " is first_tour(4, List((0, 0))) == None " >> $out

  if (scala_assert_thirty "knight2.scala" "knight2b_test.scala") 
  then
    echo "  --> success" >> $out
  else
    echo "  --> test failed" >> $out
  fi
fi

